/*
 * Copyright (c) 1998-2012 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.gen;

import com.caucho.java.gen.DependencyComponent;
import com.caucho.java.gen.GenClass;
import com.caucho.java.gen.JavaClassGenerator;
import com.caucho.make.VersionDependency;
import com.caucho.quercus.QuercusContext;
import com.caucho.quercus.program.QuercusProgram;
import com.caucho.vfs.Path;

import java.util.logging.Logger;

/**
 * Generator.
 */
public class QuercusGenerator {
  private static final Logger log
    = Logger.getLogger(QuercusGenerator.class.getName());

  private final QuercusContext _quercus;

  public QuercusGenerator(QuercusContext quercus)
  {
    _quercus = quercus;
  }

  public Class<?> preload(GenClass cl)
  {
    return preload(cl.getFullClassName());
  }


  public Class<?> preload(String className)
  {
    JavaClassGenerator gen = createGenerator();

    // preload only looks inside work dir, QuercusCompiler needs the full classpath
    //return gen.preload(className);
    
    return gen.load(className);
  }

  public boolean preloadExists(String className)
  {    
    JavaClassGenerator gen = new JavaClassGenerator();
    gen.setSearchPath(_quercus.getPwd());
    gen.setLoader(_quercus.getCompileClassLoader());
    
    return gen.preloadExists(className);
  }

  public Path getClassFilePath(QuercusProgram program)
  {
    String className = _quercus.getClassName(program.getSourcePath());

    JavaClassGenerator gen = createPreloadGenerator();

    return gen.getClassFilePath(className);
  }

  public Class<?> preload(QuercusProgram program)
    throws Exception
  {
    JavaClassGenerator gen = createPreloadGenerator();

    String className = _quercus.getClassName(program.getSourcePath());
    
    // preload only looks inside work dir, QuercusCompiler needs the full classpath
    //Class<?> pageClass = gen.preload(className);

    Class<?> pageClass = gen.load(className);
    
    return pageClass;
  }

  public Class<?> load(QuercusProgram program)
    throws Exception
  {
    JavaClassGenerator gen = createGenerator();

    String className = _quercus.getClassName(program.getSourcePath());
    
    Class<?> pageClass = gen.load(className);
    
    return pageClass;
  }

  public Class<?> preloadProfile(QuercusProgram program)
    throws Exception
  {
    JavaClassGenerator gen = createPreloadGenerator();

    String className = _quercus.getClassName(program.getSourcePath());

    Class<?> pageClass = gen.preload(className + "__prof");

    return pageClass;
  }

  public String []generate(QuercusProgram program,
                           String userPath,
                           boolean isLazy)
    throws Exception
  {
    return generate(program, userPath, isLazy, false);
  }

  public String []generateProfile(QuercusProgram program, String userPath)
    throws Exception
  {
    return generate(program, userPath, false, true);
  }

  private String []generate(QuercusProgram program,
                            String userPath,
                            boolean isLazy,
                            boolean isProfile)
    throws Exception
  {
    if (isLazy)
      program.waitForRuntimeFunctionList(2000);

    JavaClassGenerator gen = createGenerator();

    String className = _quercus.getClassName(program.getSourcePath());

    if (isProfile)
      className = className + "__prof";

    GenClass cl = new GenClass(className);

    // Java generation code
    cl.setSuperClassName("com.caucho.quercus.page.QuercusPage");

    cl.addImport("com.caucho.quercus.*");
    cl.addImport("com.caucho.quercus.classes.*");
    cl.addImport("com.caucho.quercus.env.*");
    cl.addImport("com.caucho.quercus.expr.*");
    cl.addImport("com.caucho.quercus.function.*");
    cl.addImport("com.caucho.quercus.program.*");
    cl.addImport("com.caucho.quercus.lib.*");

    QuercusMain main = new QuercusMain(program, className);
    main.setProfile(isProfile);
    main.setUserPath(userPath);

    cl.addComponent(main);

    cl.addDependencyComponent().addDependency(new VersionDependency());

    DependencyComponent dependency = cl.addDependencyComponent();
    dependency.setSearchPath(_quercus.getPwd());
    dependency.addDependencyList(program.getDependencyList());

    // gen.setEncoding("JAVA");

    gen.generate(cl);

    return gen.getPendingFiles();
  }

  public void compile(String []files)
    throws Exception
  {
    JavaClassGenerator gen = createGenerator();

    gen.addPendingFiles(files);

    gen.compilePendingJava();
  }

  private JavaClassGenerator createGenerator()
  {
    JavaClassGenerator gen = new JavaClassGenerator();
    
    gen.setLoader(_quercus.getCompileClassLoader());
    gen.setSearchPath(_quercus.getPwd());
    gen.setWorkDir(_quercus.getWorkDir());

    return gen;
  }
  
  private JavaClassGenerator createPreloadGenerator()
  {
    JavaClassGenerator gen = new JavaClassGenerator();

    gen.setLoader(_quercus.getCompileClassLoader());
    gen.setSearchPath(_quercus.getPwd());
    gen.setWorkDir(_quercus.getWorkDir());

    return gen;
  }
}

